module("Initialisation");

test("correct instanceof", function() {
    equals( issuePicker instanceof AJS.IssuePicker, true, "Expected issuePicker to be instance of AJS.IssuePicker" );
    equals( issuePicker instanceof AJS.MultiSelect, true, "Expected issuePicker to be instance of AJS.MultiSelect" );
    equals( issuePicker instanceof AJS.AbstractInlineLayer, true, "Expected issuePicker to be instance of AJS.AbstractInlineLayer" );
    equals( issuePicker instanceof AJS.Layer, true, "Expected issuePicker to be instance of AJS.AbstractLayer" );
});


module("Private methods");

test("_addItem", function() {

    expect(10);

    function testAddItem (descriptor, index) {

        var returnValue = issuePicker._addItem(descriptor),
            representations = jQuery(".multi-select .items li"),
            representation = representations.eq(index);

        equals(returnValue, undefined, "Expected return value to be undefined");

        equals(representations.length === index + 1, true, "First representaion was not added to textarea");

        equals(representation.data("descriptor"), descriptor, "Expected jQuery data 'descriptor' to be set on representation");

        if (descriptor.label.length > 12) {
            equals(representation.is(':contains("...")'), true, "Expected representation to contain '...' when longer than 6 characters");
        } else {
            equals(representation.is(':contains("...")'), false, "Expected representation not contain '...' when less than 6 characters");
        }

        equals(representation.is(":contains(" + descriptor.label.substring(0, 6) + ")"), true, "Expected representation to contain label");
    }

    testAddItem(generateMockItemDescriptor(JRA1800ItemDescriptor), 0);
    testAddItem(generateMockItemDescriptor(JRA1330ItemDescriptor), 1);
});


test("_appendOptions", function () {


    function testValidOption (index, descriptor) {

        var option = issuePicker.$model.find("option").eq(index);

        equals(option.attr("value"), descriptor.value, "Expected option 'value' to be that of descriptor 'value'");
        equals(option.attr("title"), descriptor.title, "Expected option 'title' to be that of descriptor 'title'");
        ok(option.css("backgroundImage").indexOf(descriptor.icon) !== -1, "Expected 'backgroundImage' to be that of descriptor 'icon'");
        equals(option.text(), descriptor.label, "Expected 'text' to be that of descriptor 'label'");
    }

    var returnValue = issuePicker._appendOptions([JRA1800ItemDescriptor, JRA1330ItemDescriptor]);

    equals(returnValue, undefined, "Expected return value to be undefined");
    equals(issuePicker.$model.find("option").length, 2, "Expected 2 elements to be added to <select> element");

    testValidOption(0, JRA1800ItemDescriptor);
    testValidOption(1, JRA1330ItemDescriptor);
});


test("_assignEvents", function () {

    // _assignEvents does not need to be invocted because it is called by the constructor.

    var event,
        resizableCalled,
        expectedCalls = issuePicker.$selectedItemsContainer.find(".value-item").length,
        _initItemCalledTimes = 0;

    issuePicker.$layer.click(function (e) {
        equals(e.isPropagationStopped(), true, "Expected event propagtion to be stopped on $layer");
    });

    issuePicker.$layer.mousedown(function (e) {
        equals(e.isDefaultPrevented(), true, "Expected default event to be prevented");
    });

    issuePicker.$layer.trigger("click");
    issuePicker.$layer.trigger("mousedown");

    event = jQuery.Event("keypress");
    event.keyCode = 0;

    assertEventCallsFunction(issuePicker.$field, event, {
        obj: issuePicker,
        method: "_handleCharacterInput"
    });

    event = jQuery.Event("keypress");
    event.keyCode = 13;

    assertEventCallsFunction(issuePicker.$field, event, {
        obj: issuePicker,
        method: "_handleModifiers"
    });

    assertEventFiresEvent({
        $element: issuePicker.$selectedItemsContainer,
        event: "click"
    }, {
        $element: issuePicker.$field,
        event: "focus"
    });

    assertEventCallsFunction(window, "resize", {
        obj: issuePicker,
        method: "_resetMaxHeightOfSuggestions"
    });

    jQuery.fn.resizable = function () {
        resizableCalled =true;
    };

    jQuery.aop.after({target: issuePicker, method: "_initItem"}, function () {
        ++_initItemCalledTimes;
    });

    issuePicker._assignEvents();

    equals(_initItemCalledTimes, expectedCalls, "Expected _initItem to be called " + expectedCalls + "times");

});


test("_createFurniture", function () {

    var _updateItemsIndentCalled = false;

    assertElementPresent(".multi-select");
    assertElementPresent(".representation");
    assertElementPresent(".items");
    assertElementPresent(".suggestions");
    assertElementPresent("textarea.aui-field");

    equals(issuePicker.listController instanceof AJS.List, true, "Expected issuePicker.listController to be an instance of 'AJS.List'");

    issuePicker.updateItemsIndent = function () {
        _updateItemsIndentCalled = true;
    };

    issuePicker._createFurniture();

    equals(_updateItemsIndentCalled, true, "Expected '_updateItemsIndent' to be called when creating furniture");
});

test("_filterData", function () {

    var suggestionsToBeDisplayed;

    issuePicker.availableDescriptors = [JRA1800ItemDescriptor, JRA1330ItemDescriptor];
    issuePicker.selectedDescriptors = [JRA1330ItemDescriptor];
    suggestionsToBeDisplayed = issuePicker._filterData();
    equals(suggestionsToBeDisplayed.length, 1, "Expected previous selected descriptor to be filtered");
    equals(suggestionsToBeDisplayed[0].label, JRA1800ItemDescriptor.label, "Expected availableDescriptor for display to be JIRA1800");
});

test("_generateSuggestions", function () {

    var suggestions = issuePicker._generateSuggestions({
        data: [JRA1800ItemDescriptor, JRA1330ItemDescriptor, MKY10ItemDescriptor],
        search: "o"
    });

    equals(suggestions.length, 2, "Expected 2 suggestions to be displayed");
    equals(suggestions.eq(0).find("em:contains(o)").length, 3, "Expected all 3 'o's to be bolded");
    equals(suggestions.eq(1).find("em:contains(o)").length, 1, "Expected 1 'o's to be bolded");


    suggestions = issuePicker._generateSuggestions({
        data: [JRA1800ItemDescriptor, JRA1330ItemDescriptor, MKY10ItemDescriptor],
        search: "Browse"
    });

    equals(suggestions.length, 1, "Expected 1 suggestions to be displayed");
    equals(suggestions.find("a.icn").length, 1,  "Expected JRA1800 to have an icn class");
    equals(suggestions.find("em:contains(browse)").length, 1, "Expected 'browse' to be bolded");

    suggestions = issuePicker._generateSuggestions({
        data: [JRA1800ItemDescriptor, JRA1330ItemDescriptor, MKY10ItemDescriptor],
        search: "Monkey"
    });

    equals(suggestions.length, 1, "Expected 1 suggestions to be displayed");
    equals(suggestions.find("em:contains(Monkey)").length, 1, "Expected 'monkey' to be bolded");
    equals(suggestions.find("a.icn").length, 0,  "Expected Monkey issue not to have an icn class");
    
    suggestions = issuePicker._generateSuggestions({
        data: [JRA1800ItemDescriptor, JRA1330ItemDescriptor, MKY10ItemDescriptor],
        search: "xxx"
    });

    equals(suggestions.length, 1, "Expected 1 suggestions to be displayed (No suggestion)");
    equals(suggestions.is(":contains('No suggestions')"), true, "Expected suggestions to contain the text 'No suggestions'");

});

test("_getDefaultOptions", function () {
    equals(typeof issuePicker._getDefaultOptions({}), "object", "Expected return type of Object");
});

test("_getInputIndent", function () {
    assertWithinThreshold(issuePicker._getInputIndent().left, 2);
    issuePicker._addItem(generateMockItemDescriptor(JRA1800ItemDescriptor));
    assertWithinThreshold(issuePicker._getInputIndent().left, 156);
    issuePicker._addItem(generateMockItemDescriptor(JRA1800ItemDescriptor));
    assertWithinThreshold(issuePicker._getInputIndent().left, 306);
    issuePicker._addItem(generateMockItemDescriptor(JRA1330ItemDescriptor));
    assertWithinThreshold(issuePicker._getInputIndent().left, 5);
    assertWithinThreshold(issuePicker._getInputIndent().top, 27);
    issuePicker._addItem(generateMockItemDescriptor(JRA1800ItemDescriptor));
    assertWithinThreshold(issuePicker._getInputIndent().left, 156);
    assertWithinThreshold(issuePicker._getInputIndent().top, 27);
    issuePicker._addItem(generateMockItemDescriptor(JRA1330ItemDescriptor));
    assertWithinThreshold(issuePicker._getInputIndent().left, 272);
    assertWithinThreshold(issuePicker._getInputIndent().top, 27);
});

test("_getTargetItemContainerWidth", function () {
    assertWithinThreshold(issuePicker._getTargetItemContainerWidth(), 0);
    issuePicker._addItem(generateMockItemDescriptor(JRA1800ItemDescriptor));
    assertWithinThreshold(issuePicker._getTargetItemContainerWidth(), 150);
    issuePicker._addItem(generateMockItemDescriptor(JRA1330ItemDescriptor));
    assertWithinThreshold(issuePicker._getTargetItemContainerWidth(), 266);
    issuePicker._addItem(generateMockItemDescriptor(JRA1330ItemDescriptor));
    assertWithinThreshold(issuePicker._getTargetItemContainerWidth(), 382);
    issuePicker._addItem(generateMockItemDescriptor(JRA1330ItemDescriptor));
    assertWithinThreshold(issuePicker._getTargetItemContainerWidth(), 498);
    issuePicker._addItem(generateMockItemDescriptor(JRA1800ItemDescriptor));
    assertWithinThreshold(issuePicker._getTargetItemContainerWidth(), 648);
});

test("_handleCharacterInput", function () {
    issuePicker._handleCharacterInput();
    assertEventDoesntCallsFunction(issuePicker.$field, "keyup", {
        obj: issuePicker,
        method: "_requestThenResetSuggestions"
    });

    issuePicker._handleCharacterInput();
    assertEventCallsFunction(issuePicker.$field, "keyup", {
        obj: issuePicker,
        method: "hide"
    });

    issuePicker.$field.val("t");
    issuePicker._handleCharacterInput();
    assertEventCallsFunction(issuePicker.$field, "keyup", {
        obj: issuePicker,
        method: "_requestThenResetSuggestions"
    });
});

test("_handleModifiers", function () {

    var event = jQuery.Event("keypress");
    event.keyCode = 40;
    
    assertEventCallsFunction(issuePicker.$field, event, {
        obj: issuePicker,
        method: "_handleDown"
    });

    event = jQuery.Event("keypress");
    event.keyCode = 8;

    assertEventCallsFunction(issuePicker.$field, event, {
        obj: issuePicker,
        method: "_handleBackSpace"
    });

    event = jQuery.Event("keypress");
    event.keyCode = 13;

    issuePicker.$field.keypress(function (e) {
        equals(e.isDefaultPrevented(), true, "Expected default event to be prevented");
    });

    issuePicker.$field.trigger(event);
});

test("_handleBackspace", function () {

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_handleBackSpace"
    }, {
        obj: issuePicker,
        method: "selectLastItem"
    });

    issuePicker.$field.val("a");


    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_handleBackSpace"
    }, {
        obj: issuePicker,
        method: "_handleCharacterInput"
    });

});

test("_handleDown", function () {

    var event = jQuery.Event("keypress");
    event.keyCode = 40;

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_handleDown",
        arguments: [event]
    }, {
        obj: issuePicker,
        method: "show"
    });

    issuePicker.$layer.show();

    assertMethodDoesntCallsMethod({
        obj: issuePicker,
        method: "_handleDown",
        arguments: [event]
    }, {
        obj: issuePicker,
        method: "show"
    });

});

test("_initItem", function () {

    var event = jQuery.Event("keypress");
    event.keyCode = 37;

    issuePicker._initItem(itemButton);

    assertEventCallsFunction(itemButton, event, {
        obj: issuePicker,
        method: "_itemLeftHandler"
    });

    event = jQuery.Event("keypress");
    event.keyCode = 9;

    assertEventCallsFunction(itemButton, event, {
        obj: issuePicker,
        method: "_itemTabHandler"
    });

    event = jQuery.Event("keypress");
    event.keyCode = 8;

    assertEventCallsFunction(itemButton, event, {
        obj: issuePicker,
        method: "_itemBackSpaceHandler"
    });

    assertEventCallsFunction(itemButton, "focus", {
        obj: issuePicker,
        method: "hide"
    });

    assertEventFiresEvent({
        $element: itemButton,
        event: "blur"
    }, {
        $element: issuePicker.$field,
        event: "focus"
    });

    assertEventCallsFunction(itemButton, "click", {
        obj: issuePicker,
        method: "removeItem"
    });

});

test("_itemBackSpaceHandler", function () {

    var event = jQuery.Event("keypress");

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_itemBackSpaceHandler",
        arguments: [event, itemButton]
    }, {
        obj: issuePicker,
        method: "removeItem"
    });

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_itemBackSpaceHandler",
        arguments: [event, itemButton]
    }, {
        obj: issuePicker,
        method: "selectLastItem"
    });

});

test("_itemTabHandler", function () {

    var event = jQuery.Event("keypress");

    asserMethodCallsEvent({
        obj: issuePicker,
        method: "_itemTabHandler",
        arguments: [event]
    }, {
        $element: issuePicker.$field,
        event: "focus"
    });
});


test("_itemLeftHandler", function () {

    var event = jQuery.Event("keypress"),
        itemButtonClone;

    asserMethodCallsEvent({
        obj: issuePicker,
        method: "_itemLeftHandler",
        arguments: [event, itemButton]
    }, {
        $element: issuePicker.$field,
        event: "focus"
    });

    itemButtonClone = itemButton.clone();
    jQuery("<li />").append(itemButtonClone).appendTo(issuePicker.$selectedItemsContainer);
    jQuery("<li />").append(itemButton).appendTo(issuePicker.$selectedItemsContainer);

    asserMethodCallsEvent({
        obj: issuePicker,
        method: "_itemLeftHandler",
        arguments: [event, itemButton]
    }, {
        $element: itemButtonClone,
        event: "focus"
    });
});

test("_parseDescriptors", function () {

    var JRA1800, JRA1330;

    issuePicker.$model.append(generateMockItemDescriptor(JRA1800ItemDescriptor).model);
    issuePicker.$model.append(generateMockItemDescriptor(JRA1330ItemDescriptor).model);
    
    issuePicker._parseDescriptors();

    JRA1800 = issuePicker.availableDescriptors[0];

    equals(JRA1800.label, JRA1800ItemDescriptor.label, "Expected parsed descriptor to be same as mock descriptor");
    equals(JRA1800.value, JRA1800ItemDescriptor.value, "Expected parsed descriptor to be same as mock descriptor");
    equals(JRA1800.title, JRA1800ItemDescriptor.title, "Expected parsed descriptor to be same as mock descriptor");

    JRA1330 = issuePicker.availableDescriptors[1];

    equals(JRA1330.label, JRA1330ItemDescriptor.label, "Expected parsed descriptor to be same as mock descriptor");
    equals(JRA1330.value, JRA1330ItemDescriptor.value, "Expected parsed descriptor to be same as mock descriptor");
    equals(JRA1330.title, JRA1330ItemDescriptor.title, "Expected parsed descriptor to be same as mock descriptor");
    
});


test("_requestThenResetSuggestions", function () {

    var aborted;

    issuePicker.latestRequest = {abort: function () {
        aborted = true;
    }};

    issuePicker._requestThenResetSuggestions();

    ok(aborted, "Expected to abort previous xhr request when a new one starts");

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_requestThenResetSuggestions"
    }, {
        obj: issuePicker.$model,
        method: "empty"
    });

});

test("_handleSuggestionResponse", function () {
    var called;
    issuePicker.options.ajaxOptions.formatResponse = function () {
        called = true;
    };
    issuePicker._handleSuggestionResponse({});
    ok(called, "Expected options.formatResponse to be called");
});


test("_resetSuggestions", function () {
    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_resetSuggestions"
    },
    [
        {
            obj: issuePicker,
            method: "_generateSuggestions"
        },
        {
            obj: issuePicker,
            method: "_resetMaxHeightOfSuggestions"
        },
        {
            obj: issuePicker,
            method: "_setSuggestionsWidth"
        },
        {
            obj: issuePicker.listController,
            method: "reset"
        }
    ]);
});

test("_selectionHandler", function () {

    issuePicker.$field.val("test");

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "_selectionHandler",
        arguments: [suggestedOptionMock]
    },  
    {
        obj: issuePicker,
        method: "_addItem"
    });

    equals(issuePicker.$field.val(), "", "Expected field value to be empty");
});

test("_setSuggestionsWidth", function () {

    issuePicker.$layer.show();

    issuePicker.$layerContent.width(10);
    issuePicker._setSuggestionsWidth();
    assertWithinThreshold(issuePicker.$layerContent.width(), 252);

    issuePicker.$layerContent.html("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
    issuePicker._setSuggestionsWidth();
    assertWithinThreshold(issuePicker.$layerContent.width(), 500);
});

test("updateItemsIndent", function () {

    var indents = [69, 132, 195, 258, 321, 384, 5, 69, 132, 195],
        topPadding =  [5, 5, 5, 5, 5, 5, 22, 27, 27, 27];

    for (var i=0; i < 10; i++) {
        generateMockLI().appendTo(issuePicker.$selectedItemsContainer);
        issuePicker.selectedItems = jQuery("li", issuePicker.$selectedItemsContainer);
        issuePicker.updateItemsIndent();
        assertWithinThreshold(parseInt(issuePicker.$field.css("textIndent"), 10), indents[i]);
        assertWithinThreshold(parseInt(issuePicker.$field.css("paddingTop"), 10), topPadding[i]);
    }
});

test("_updateModel", function () {

    var availableDescriptors = generateMockItemDescriptor(JRA1800ItemDescriptor);

    generateMockLI().data("descriptor", availableDescriptors)
            .appendTo(issuePicker.$selectedItemsContainer);


    issuePicker._updateModel();

    equals(issuePicker.selectedDescriptors.length, 1, "Expecteded only one item to be selected");
    equals(issuePicker.selectedDescriptors[0].model.attr("selected"), true, "Expecteded item in selected items container to be selected in model also");

    issuePicker.$selectedItemsContainer.find("li").remove();

    issuePicker._updateModel();

    equals(issuePicker.selectedDescriptors.length, 0, "There should be no items selected");

});

module("Public methods");

test("hide", function () {

    assertMethodDoesntCallsMethod({
        obj: issuePicker,
        method: "hide"
    },
    {
        obj: issuePicker.listController,
        method: "disable"
    });

    issuePicker.$layer.show();

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "hide"
    },
    {
        obj: issuePicker.listController,
        method: "disable"
    });

});

test("show", function () {

    issuePicker.$layer.show();

    assertMethodDoesntCallsMethod({
        obj: issuePicker,
        method: "show"
    },
    [
        {
            obj: issuePicker.listController,
            method: "enable"
        },
        {
            obj: issuePicker,
            method: "_resetSuggestions"
        },
        {
            obj: issuePicker,
            method: "updateItemsIndent"
        }
    ]);

    issuePicker.$layer.hide();

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "show"
    },
    [
        {
            obj: issuePicker.listController,
            method: "enable"
        },
        {
            obj: issuePicker,
            method: "_resetSuggestions"
        },
        {
            obj: issuePicker,
            method: "updateItemsIndent"
        }
    ]);
});

test("removeItem", function () {

    var descriptor = generateMockItemDescriptor(MKY10ItemDescriptor),
        li = generateMockLI()
            .data("descriptor", descriptor)
            .appendTo(issuePicker.$selectedItemsContainer);

    descriptor.model.attr("selected", "selected").appendTo(issuePicker.$model);

    issuePicker.selectedItems = jQuery("li", issuePicker.$selectedItemsContainer);

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "removeItem",
        arguments: [li.find("button")]
    },
    [
        {
            obj: issuePicker,
            method: "updateItemsIndent"
        },
        {
            obj: issuePicker,
            method: "_updateModel"
        }
    ]);

    ok(li.parent().length === 0, "Expected li to be removed from selectedItemsContainer");
    ok(issuePicker.selectedItems.length === 0, "Expected item to be removed from selectedItems");
});

test("selectLastItem", function () {

    var descriptor = generateMockItemDescriptor(MKY10ItemDescriptor);

    assertMethodCallsMethod({
        obj: issuePicker,
        method: "selectLastItem"
    },
    {
        obj: issuePicker,
        method: "updateItemsIndent"
    });

    assertMethodFiresEvent({
        obj: issuePicker,
        method: "selectLastItem"
    },
    {
        event: "focus",
        $element: issuePicker.$field
    });

    generateMockLI().data("descriptor", descriptor).appendTo(issuePicker.$selectedItemsContainer);
    issuePicker.selectedItems = jQuery("li", issuePicker.$selectedItemsContainer);

    assertMethodDoesntCallsMethod({
        obj: issuePicker,
        method: "selectLastItem"
    },
    {
        obj: issuePicker,
        method: "updateItemsIndent"
    });
    
    generateMockLI().data("descriptor", descriptor).appendTo(issuePicker.$selectedItemsContainer);
    issuePicker.selectedItems = jQuery("li", issuePicker.$selectedItemsContainer);


    assertMethodFiresEvent({
        obj: issuePicker,
        method: "selectLastItem"
    },
    {
        event: "focus",
        $element: AJS.$("button:last", issuePicker.$selectedItemsContainer)
    });

});

